"""
QMonination Helper

@author Shubham Chaudhary
@email me@shubhamchaudhary.in
"""
from pkg_resources import get_distribution, DistributionNotFound


__title__ = 'qmon - Redis Monitor'
__author__ = 'Shubham Chaudhary'
__license__ = 'GPLv3+'
__copyright__ = 'Copyright 2017 Shubham Chaudhary'
try:
    __version__ = get_distribution('qmon').version
except DistributionNotFound:
    __version__ = '0.0.1'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import logging
import os
import time

import hypchat

import qmon.status

logger = logging.getLogger(__name__)
TIME_GAP_IN_SECONDS = 5 * 60
APPROX_QUEUE_SIZE = 1e10
NUM_ITEMS_BEFORE_NOTIFY_TOLERANCE = 500


def notify(message, items_in_queue, last_notify_items_length, room):
    def __should_i_notify_subscribers_now():
        return items_in_queue - last_notify_items_length > NUM_ITEMS_BEFORE_NOTIFY_TOLERANCE

    should_notify = __should_i_notify_subscribers_now()
    # echo $MSG | mutt -s "Ratings Queue Monitoring - $ITEMS" shubham.chaudhary@zomato.com

    room.notification(
        message,
        color='green' if should_notify else 'gray',  # yellow, green, red, purple, gray, random
        notify=should_notify
    )
    logger.info('{:s}. Items: {:,d}. Last Notify: {:,d}'.format(message, items_in_queue, last_notify_items_length))
    return should_notify


def monitor_queue(queue_name, host, port, room):
    logger.info('Connecting to hipchat')
    hc = hypchat.HypChat(os.environ['HIP_TOKEN'])
    room = hc.get_room(room)

    room.notification(
        'Starting to monitor queue={queue} on port={port}'.format(queue=queue_name, port=port),
        color='yellow',  # yellow, green, red, purple, gray, random
        notify=True
    )

    items_in_queue = APPROX_QUEUE_SIZE
    last_notify_items_length = 0
    last_items_length = 0
    rd = qmon.status.redis_connection_cached(host, port)
    while items_in_queue:
        items_in_queue = qmon.status.get_items_in_queue(queue_name, redis_conn=rd, default=0)
        if last_items_length != items_in_queue:
            message = '{} queue status: {:,d}'.format(queue_name, items_in_queue)

            notified = notify(message, items_in_queue, last_notify_items_length, room)
            if notified:
                last_notify_items_length = items_in_queue
            last_items_length = items_in_queue
        time.sleep(TIME_GAP_IN_SECONDS)

    room.notification(
        'All done for {queue}.'.format(queue=queue_name),
        color='yellow',  # yellow, green, red, purple, gray, random
        notify=True
    )



# -*- coding: utf-8 -*-
from redis import StrictRedis

_redis = None


def redis_connection_cached(host, port):
    global _redis
    if _redis is None:
        _redis = redis_connection(host=host, port=port)
    return _redis


def redis_connection(host, port):
    return StrictRedis(host=host, port=port)


def get_items_in_queue(queue_name, redis_conn, default=None):
    type_name = redis_conn.type(queue_name).decode()
    if type_name == 'hash':
        length_op = redis_conn.hlen
    elif type_name == 'list':
        length_op = redis_conn.llen
    elif type_name == 'set':
        length_op = redis_conn.scard
    elif type_name == 'none' and default is not None:
        length_op = lambda _: default
    else:
        raise ValueError('Queue empty or Unknown type_name "{}" of queue "{}"'.format(type_name, queue_name))
    return length_op(queue_name)



#!/usr/bin/env python3
#
#   main.py - All commandline interactions
#   This file is a part of QMon.
#
#   Copyright (c) 2017 Shubham Chaudhary <me@shubhamchaudhary.in>
#
#   QMon is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   QMon is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with QMon. If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import print_function

import logging
from argparse import ArgumentParser

import qmon.monitor
from qmon import __version__


def print_version():
    print('QMon version %s' % __version__)
    print('Copyright (c) 2017 by Shubham Chaudhary.')
    print('License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>')
    print('This is free software: you are free to change and redistribute it.')
    print('There is NO WARRANTY, to the extent permitted by law.')


def parse_known_args():
    """ Parse command line arguments """
    parser = ArgumentParser()
    parser.add_argument('--queue-name', '-q', required=True, help='Name of queue')
    parser.add_argument('--host', required=True, help='Redis host')
    parser.add_argument('--port', '-p', help='Redis port')
    parser.add_argument('--room', '-r', help='HipChat Room')
    parser.add_argument('-V', '--version',
                        action='store_true',
                        dest='version',
                        help='Print the version number and exit')
    args, otherthings = parser.parse_known_args()
    return args, otherthings, parser


def main():
    args, otherthings, parser = parse_known_args()

    setup_logging()
    qmon.monitor.monitor_queue(queue_name=args.queue_name, host=args.host, port=args.port, room=args.room)
    return 0


def setup_logging():
    logging.basicConfig(format='[%(name)s] [%(asctime)s] %(levelname)s : %(message)s')
    logging.getLogger('root').setLevel(logging.DEBUG)
    logging.getLogger('qmon').setLevel(logging.DEBUG)
    logging.getLogger('qmon.monitor').setLevel(logging.DEBUG)


if __name__ == '__main__':
    import sys

    setup_logging()
    sys.exit(main())
